// Filename: LLISTFUN.H
// Purpose:  Header file of definitions and functions to be used
//           with the demonstration program in LNODEDRV.CPP.


#include <iostream>
#include <iomanip>
using namespace std;

struct Node
{
	//Your struct body goes here
};
typedef Node* NodePointer;

/*
Part of the documentation of each of the following functions is a
list of what functions are called by that function and/or what functions
that function calls. Unless otherwise indicated, any function missing this
documentation is called only by main and does not call any other functions.
*/


/*
The following three functions form the driver "program infrastructure",
but have nothing to do with sequences of linked nodes.
*/

void DescribeProgram();
// Pre:  none
// Post: The program description has been displayed.

void DisplayMenu();
// Pre:  none
// Post: A menu with 16 options has been displayed.

void GetMenuChoice(/* out */ int& menuChoice);
// Pre:  none
// Post: "menuChoice" contains an integer entered by the user
//       from the keyboard



/*
The following three functions are "utility" functions that initialize
a sequence, test if one is empty, and reclaim the dynamic storage alloted
to a sequence when that sequence is no longer needed.
*/

void InitializeSequence(/* inout */ NodePointer& head);
// Pre:  none
// Post: "head" has been set to NULL.
// Called by: BuildSequenceFixedSize, BuildSequenceAnySize

bool EmptySequence(/* in */ NodePointer head);
// Pre:  "head" has been initialized.
// Post: The function value is true if the sequence is empty,
//       and false otherwise.
// Called by: BuildSequenceFixedSize, BuildSequenceAnySize

void ReclaimStorage(/* in */ NodePointer head);
// Pre:  "head" has been initialized.
// Post: All storage (if any) in the node sequence pointed to by
//       "head" has been returned to the free store.
// Called by: BuildSequenceFixedSize, BuildSequenceAnySize



/*
The next function allow you to build a sequence of fixed size.
*/

void BuildSequenceFixedSize(/* inout */ NodePointer& head);
// Pre:  none
// Post: The user has entered the number of nodes to be in a
//       newly constructed sequence, as well as the values to
//       be inserted into the sequence, the sequence has been
//       constructed, and "head" points at the first node.
//       All storage in any sequence previously pointed to by
//       "head" has been returned to the free store.



/*
This function allows you to display the values in any sequence.
*/

void PrintNodeValues(/* inout */ NodePointer& head);
// Pre:  "head" has been initialized.
// Post: If the sequence is empty, a message to that effect has
//       been output.  Otherwise, all values in the sequence
//       pointed to by "head" have been displayed, ten to a line,
//       with each one in a field width of six spaces.
// Calls: InitializeSequence, EmptySequence, ReclainStorage



/*
The following four functions are "utility functions" that are called by
other functions to do part of their work.

The first two find (respectively) the node containing a data value, and
that node plus the previous node.  The previous node is needed if we plan
to insert before a node or delete a node.

The next two find (respectively) the node having a given position, and
that node plus the previous node.  Once again, the previous node is needed
if we plan to insert before a node or delete a node.
*/

void FindNodeDataValue(/* inout */ NodePointer   head,
                       /* in */    DataType      targetValue,
                       /* out */   bool&         found,
                       /* out */   NodePointer&  targetPointer,
                       /* out */   int&          targetPosition);
// Pre:  "head" and "targetValue" have been initialized.
// Post: If "targetValue" is in the sequence, "found" is true and:
//       - "targetPointer" points to the first node of the sequence
//         containing the "targetValue"
//       - "targetPosition" contains the number of the first node
//         in the sequence containing the "targetValue"
//       Otherwise, "found" is false and both "targetPointer" and
//       "targetPosition" are undefined.
// Called by: FindDataValues, InsertAfterNodeWithDataValue

void FindDataValueAndPrevious(/* inout */ NodePointer&  head,
                              /* in */    DataType      targetValue,
                              /* out */   bool&         found,
                              /* out */   NodePointer&  targetPointer,
                              /* out */   NodePointer&  previousPointer);
// Pre:  "head" and "targetValue" have been initialized.
// Post: If "targetValue" is in the sequence, "found" is true and
//       "targetPointer" points at the first node of the sequence
//       containing "targetValue", with "previousPointer" pointing
//       at the previous node, unless "targetPointer" is pointing at
//       the first node, in which case the value of "previousPointer"
//       is NULL.  Otherwise, "found" is false and the values of both
//       "targetPointer" and "previousPointer" are undefined.
// Called by: InsertBeforeNodeWithDataValue, DeleteNodeWithDataValue


void FindNodeK(/* inout */ NodePointer&  head,
               /* in */    int           k,
               /* out */   bool&         found,
               /* out */   NodePointer&  targetPointer);
// Pre:  "head" and "k" have been initialized.
// Post: If the sequence contains k or more nodes, "found" is true
//       and "targetPointer" points to node k.  Otherwise, "found"
//       is false and "targetPointer" is undefined.
// Called by: FindPositions, InsertAfterNodeK

void FindNodeKAndPrevious(/* inout */ NodePointer&  head,
                          /* in */    int           k,
                          /* out */   bool&         found,
                          /* out */   NodePointer&  targetPointer,
                          /* out */   NodePointer&  previousPointer);
// Pre:  "head" and "k" have been initialized.
// Post: If the sequence has k or more nodes, "found" is true and
//       "targetPointer" points at node k of the sequence, with
//       "previousPointer" pointing at the previous node, unless
//       node k is the first node, in which case the value of
//       "previousPointer" is NULL.  Otherwise, "found" is false
//       and the values of both "targetPointer" and "previousPointer"
//       are undefined.
// Called by: InsertBeforeNodeK, DeleteNodeK



/*
The next two functions find a particular node, either via the data value
in it, or the node's position.
*/
void FindDataValues(/* inout */ NodePointer& head);
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more integer values and
//       in each case a message has been displayed, indicating
//       either the first position in the sequence where that
//       value was located or that the value was not found in
//       in the current sequence.
// Calls: FindNodeDataValue

void FindPositions(/* inout */ NodePointer& head);
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more integer position
//       numbers and in each case a message has been displayed,
//       indicating either the value found in that position or
//       that position was not found in the current sequence.
// Calls: FindNodeK



/*
The next two functions find a node (via data value or position) and
insert a new node after that node.
*/
void InsertAfterNodeWithDataValue(/* inout */ NodePointer& head);
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more data value pairs, and
//       in each case the second value in the pair has been inserted
//       into the sequence after the first occurrence of the first
//       value in the sequence, or a message has been displayed
//       indicating that the first value of the pair was not found
//       in the current sequence.
// Calls: FindNodeDataValue

void InsertAfterNodeK(/* inout */ NodePointer& head);
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more position/value pairs,
//       and in each case the value in the pair has been inserted
//       into the sequence after the given position, or a message
//       has been displayed indicating that the given position was
//       not found in the current sequence.
// Calls: FindNodeK



/*
The next two functions find a node (via data value or position) and
insert a new node before that node.
*/
void InsertBeforeNodeWithDataValue(/* inout */ NodePointer& head);
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more data value pairs, and
//       in each case the second value in the pair has been inserted
//       into the sequence before the first occurrence of the first
//       value in the sequence, or a message has been displayed
//       indicating that the first value of the pair was not found
//       in the current sequence.
// Calls: FindDataValueAndPrevious

void InsertBeforeNodeK(/* inout */ NodePointer& head);
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more position/value pairs,
//       and in each case the value in the pair has been inserted
//       into the sequence before the given position, or a message
//       has been displayed indicating that the given position was
//       not found in the current sequence.
// Calls: FindeNodeKAndPrevious



/*
The next two functions find a node (via data value or position) and
then delete that node.
*/
void DeleteNodeWithDataValue(/* inout */ NodePointer& head);
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more data values and in
//       each case the first node of the sequence containing
//       that value has been deleted from the sequence and its
//       storage has been returned to the free store, or a
//       message has been displayed indicating that the value
//       was not found in the current sequence.
// Calls: FindDataValueAndPrevious

void DeleteNodeK(/* inout */ NodePointer& head);
// Pre:  "head" has been initialized.
// Post: The user has entered zero or more node positions and
//       in each case the node of the sequence having the given
//       position has been deleted from the sequence and its
//       storage has been returned to the free store, or a
//       message has been displayed indicating that the given
//       position was not found in the current sequence.
// Calls: FindeNodeKAndPrevious

